import argparse
from typing import List

def bounce_in_gt(ground_truth: List[str], bounce_pos: int, radius: int = 1) -> bool:
    return 'b' in ground_truth[max(bounce_pos - radius, 0) : bounce_pos + radius + 1]

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("ground_truth", type=argparse.FileType())
    parser.add_argument("out_file", type=argparse.FileType())
    parser.add_argument("-r", "--radius", help="Allowed margin of error for bounce detection", type=int, default=3)
    args = parser.parse_args()

    ground_truth = ''.join(x[:-1] for x in args.ground_truth)

    false_pos = 0
    true_pos = 0

    for i, detection in enumerate(args.out_file):
        if 'b' in detection:
            if bounce_in_gt(ground_truth, i, radius=args.radius):
                f = ground_truth.find('b', max(i - args.radius, 0), i + args.radius + 1)
                ground_truth = ground_truth[:f] + 'q' + ground_truth[f+1:]
                true_pos += 1
            else:
                false_pos += 1

    false_neg = ground_truth.count('b')

    true_neg = len(ground_truth) - false_neg - true_pos - false_pos
    
    print("truepos", true_pos)
    print("falsepos", false_pos)
    print("falseneg", false_neg)
    print("trueneg", true_neg)


if __name__ == "__main__":
    main()