//
// Created by jordi on 11/02/18.
//

#include <opencv2/cudaimgproc.hpp>
#include <iostream>
#include <opencv2/features2d.hpp>
#include <opencv/cv.hpp>
#include "BallDetector.h"
#include "HistoryElem.h"

BallDetector::BallDetector() {
    SimpleBlobDetector::Params params;
    params.blobColor = 255;
    params.filterByColor = true;
    params.filterByInertia = false;
    params.filterByConvexity = false;
    params.minCircularity = 0.7;
    params.filterByCircularity = true;
    params.maxArea = 200;
    params.minArea = 10;
    params.filterByArea = true;
    params.thresholdStep = 20;
    params.minDistBetweenBlobs = 100;
    blobDetector = SimpleBlobDetector::create(params);
}

Point2f BallDetector::detect(cv::Mat input, bool fullDetection) {
    if (fullDetection) {
        std::vector<KeyPoint> keypoints;
        blobDetector->detect(input, keypoints);
        if (keypoints.empty())
            return NULL_POINT;
        return keypoints.front().pt;
    } else {
        std::vector<std::vector<cv::Point>> results;
        std::vector<Point2f> centers;
        std::vector<double> areas;
        findContours(input, results, RETR_EXTERNAL, CHAIN_APPROX_SIMPLE);
        if (results.empty())
            return NULL_POINT;
        centers.resize(results.size());
        areas.resize(results.size());
        std::transform(results.begin(), results.end(), areas.begin(),
                       [](cv::InputArray in) { return cv::contourArea(in); });
        auto max_elem_index = std::distance(areas.begin(), std::max_element(areas.begin(), areas.end()));
        auto contour = results[max_elem_index];
        auto moments = cv::moments(contour);
        if (moments.m00 <= 0)
            return NULL_POINT;
        return {static_cast<float>(moments.m10 / moments.m00), static_cast<float>(moments.m01 / moments.m00)};
    }
}